package net.jaekl.cfb;

// Copyright (C) 2015 Christian Jaekl

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.concurrent.ConcurrentHashMap;

import net.jaekl.qd.QDBundleFactory;

public class CfbBundle {
	public static final String ANALYSIS_FAILED = "analysis.failed";
	public static final String ANALYZED_AT = "analyzed.at";
	public static final String BUG_CATEGORY_UNKNOWN = "bug.category.unknown";
	public static final String BUG_TYPE_UNKNOWN = "bug.type.unknown";
	public static final String CANNOT_CONNECT = "cannot.connect.to.db";
	public static final String CANNOT_EXEC = "cannot.exec";
	public static final String CANNOT_SEND_MAIL = "cannot.send.mail";
	public static final String CFB = "cfb";
	public static final String CFB_MAIL_SUBJECT = "cfb.mail.subject";
	public static final String CFB_REPORT = "cfb.report";
	public static final String COMPARING_RUNS = "comparing.runs";
	public static final String COMPARING_VERSIONS = "comparing.versions";
	public static final String FIXED_BUGS = "fixed.bugs";
	public static final String NEW_BUGS = "new.bugs";
	public static final String NEW_VERSION = "new.version";
	public static final String NUM_BUGS = "num.bugs";
	public static final String NUM_BUGS_OLD = "num.bugs.old";
	public static final String OLD_BUGS = "old.bugs";
	public static final String OLD_VERSION = "old.version";
	public static final String STDERR_WAS = "stderr.was";
	public static final String STDOUT_WAS = "stdout.was";
	public static final String VERSION_NUM = "version.num";
	
	final static String BUNDLE_NAME = "cfb";
	
	static ConcurrentHashMap<Locale, CfbBundle> m_bundleMap = new ConcurrentHashMap<Locale, CfbBundle>();
	
	ResourceBundle m_bundle;
	
	public static CfbBundle getInst(Locale locale) {
		CfbBundle result = m_bundleMap.get(locale);
		if (null == result) {
			synchronized(CfbBundle.class) {
				result = m_bundleMap.get(locale);
				if (null == result) {
					result = new CfbBundle(locale); 
				}
				m_bundleMap.put(locale, result);
			}
		}
		return result;
	}
	
	private CfbBundle(Locale locale) {
		m_bundle = QDBundleFactory.getInst().getBundle(BUNDLE_NAME, locale); 
	}
	
	public String get(String key, Object... arguments) {
		try {
			if (null != m_bundle) {
				String pattern = m_bundle.getString(key);
				return MessageFormat.format(pattern, arguments);
			}
		}
		catch (MissingResourceException exc) {
			// Make it clear that something has gone wrong.
			exc.printStackTrace();	
			// Fall through to the fallback behaviour below
		}
		
		StringBuilder sb = new StringBuilder("[" + key + "]");
		for (Object obj : arguments) {
			sb.append("[" + obj + "]");
		}
		return sb.toString();
	}
}
