package net.jaekl.cfb;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import net.jaekl.qd.util.FileIO;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

public class Config {
	private static final String DB_HOST = "db.host";
	private static final String DB_NAME = "db.name";
	private static final String DB_PASS = "db.pass";
	private static final String DB_PORT = "db.port";
	private static final String DB_USER = "db.user";
	private static final String FINDBUGS_HOME = "FindBugsHome";
	private static final String MAIL_FROM = "mail.from";
	private static final String MAIL_SMTP_HOST = "mail.smtp.host";
	private static final String NOTIFY = "notify";
	
	File m_configProperties;

	String m_dbName; // db name
	File m_fbp;		// FindBugsProject file
	File m_fbDir;	// Directory where FindBugs is installed
	String m_host;	// db host
	int m_port;		// db port
	String m_user;	// db user
	String m_pass;	// db password
	String m_buildNum; // build number (version)
	boolean m_removeSchema;	// purge DB schema
	File m_output;	// File to which we should write our output (report)
	
	String       m_mailFrom;
	String       m_mailSmtpHost;
	List<String> m_notify;
	
	Options m_options;
	
	public Config() {
		m_dbName = "CFB";
		m_fbp    = null;
		m_fbDir  = null;
		m_host = "localhost";
		m_port = 5432;
		m_pass = "";
		m_user = "user";
		m_buildNum = null;
		m_removeSchema = false;
		m_output = null;
		
		m_mailFrom = "findbugs@localhost";
		m_mailSmtpHost = "localhost";
		m_notify = new ArrayList<String>();
		
		m_options = createOptions();
	}
	
	public String getMailFrom() { return m_mailFrom; }
	public String getMailSmtpHost() { return m_mailSmtpHost; }
	public ArrayList<String> getNotify() { return new ArrayList<String>(m_notify); }
	
	public String getDbHost() { return m_host; }
	public int    getDbPort() { return m_port; }
	public String getDbName() { return m_dbName; }
	public String getDbUser() { return m_user; }
	public String getDbPass() { return m_pass; }
	
	public void setDbHost(String value) { m_host = value; }
	public void setDbPort(int value)    { m_port = value; }
	public void setDbName(String value) { m_dbName = value; }
	public void setDbUser(String value) { m_user = value; }
	public void setDbPass(String value) { m_pass = value; }
	
	public void readFile(File configProperties) throws IOException
	{
		Properties props = new Properties();
		InputStream is = null;
		
		try {
			is = FileIO.getInst().openInput(configProperties);
			props.load(is);
		}
		finally {
			if (null != is) {
				is.close();
			}
		}
		
		if (props.containsKey(DB_HOST)) {
			m_host = props.getProperty(DB_HOST);
		}
		if (props.containsKey(DB_NAME)) {
			m_dbName = props.getProperty(DB_NAME);
		}
		if (props.containsKey(DB_PASS)) {
			m_pass = props.getProperty(DB_PASS);
		}
		if (props.containsKey(DB_PORT)) {
			m_port = Integer.parseInt(props.getProperty(DB_PORT));
		}
		if (props.containsKey(DB_USER)) {
			m_user = props.getProperty(DB_USER);
		}
		if (props.containsKey(FINDBUGS_HOME)) {
			m_fbDir = new File(props.getProperty(FINDBUGS_HOME));
		}
		if (props.containsKey(MAIL_FROM)) {
			m_mailFrom = props.getProperty(MAIL_FROM);
		}
		if (props.containsKey(MAIL_SMTP_HOST)) {
			m_mailSmtpHost = props.getProperty(MAIL_SMTP_HOST);
		}
		if (props.containsKey(NOTIFY)) {
			String[] addresses = props.getProperty(NOTIFY).split(",");
			m_notify = Arrays.asList(addresses);
		}
	}
	
	public Options createOptions() {
		Options opt = new Options();
		
		opt.addOption("c",  "config",      true,  "Properties configuration file");
		opt.addOption("d",  "dbname",      true,  "DB name");
		opt.addOption(null, "drop-tables", false, "Remove database schema (drop all data)");
		opt.addOption("f",  "fbp",         true,  "FindBugsProject file");
		opt.addOption("h",  "host",        true,  "DB hostname");
		opt.addOption("n",  "number",      true,  "Build number (version)");
		opt.addOption("o",  "outfile",     true,  "Output report filename");
		opt.addOption("p",  "pass",        true,  "DB password");
		opt.addOption("t",  "port",        true,  "DB port");
		opt.addOption("u",  "user",        true,  "DB username");
		
		return opt;
	}
	
	public void parseArgs(PrintWriter pw, String[] args) throws ParseException
	{
		assert(null != m_options);
		
		CommandLine line = new GnuParser().parse(m_options, args);
		if (line.hasOption("c")) {
			m_configProperties = new File(line.getOptionValue("c"));
		}
		if (line.hasOption("d")) {
			m_dbName = line.getOptionValue("d");
		}
		if (line.hasOption("f")) {
			m_fbp = new File(line.getOptionValue("f"));
		}
		if (line.hasOption("h")) {
			m_host = line.getOptionValue("h");
		}
		if (line.hasOption("n")) {
			m_buildNum = line.getOptionValue("n");
		}
		if (line.hasOption("o")) {
			m_output = new File(line.getOptionValue("o"));
		}
		if (line.hasOption("p")) {
			m_pass = line.getOptionValue("p");
		}
		m_removeSchema = line.hasOption("drop-tables");
		if (line.hasOption("t")) {
			m_port = Integer.parseInt(line.getOptionValue("t"));
		}
		if (line.hasOption("u")) {
			m_user = line.getOptionValue("u");
		}
	}
}
